import dbus
import dbus.service
import subprocess
import time
import dbus.mainloop.glib
from gi.repository import GLib

class TestService(dbus.service.Object):
    def __init__(self):
        bus_name = dbus.service.BusName('org.freedesktop.vicharakBt', bus=dbus.SystemBus())
        dbus.service.Object.__init__(self, bus_name, '/org/freedesktop/vicharakBt')

    @dbus.service.method('org.freedesktop.vicharakBt.Interface')
    def Hello(self):
        print(f"==  Hello from vicharakBt_service.py ===")

        return "Hello === Function from vicharakBt_service.py ==="

    @dbus.service.method('org.freedesktop.vicharakBt.Interface')
    def PowerOn(self):
        try:
            result = subprocess.run(['sudo', 'systemctl', 'restart', 'vicharak-bluetooth.service'], capture_output=True, text=True)
            # Write output (stdout and stderr) to the log file
            print(f"STDOUT: {result.stdout}\n")
            print(f"STDERR: {result.stderr}\n")

            if result.returncode == 0:
                print("Successfully restarted vicharak-bluetooth service")
            else:
                print(f"Failed to restart vicharak-bluetooth service: {result.stderr}")
                        
        except Exception as e:
            print(f"Error managing vicharak-bluetooth service: {e}")

        return "Hi, I am PowerON"

    @dbus.service.method('org.freedesktop.vicharakBt.Interface')
    def PowerOff(self):
        try:
            result = subprocess.run(['killall','-q','SIGTERM','rtk_hciattach'], capture_output=True, text=True)
            # Write output (stdout and stderr) to the log file
            print(f"STDOUT: {result.stdout}\n")
            print(f"STDERR: {result.stderr}\n")

            if result.returncode == 0:
                print("Successfully restarted vicharak-bluetooth service")
            else:
                print(f"Failed to restart vicharak-bluetooth service: {result.stderr}")
                       
            rfkill_result = subprocess.run(['/usr/sbin/rfkill', 'block', 'bluetooth'], capture_output=True, text=True)

            # Log output from rfkill command
            print(f"RFKILL STDOUT: {rfkill_result.stdout}\n")
            print(f"RFKILL STDERR: {rfkill_result.stderr}\n")

            if rfkill_result.returncode == 0:
                print("Successfully blocked Bluetooth")
            else:
                print(f"Failed to block Bluetooth: {rfkill_result.stderr}")
        
        except Exception as e:
            print(f"Error managing vicharak-bluetooth service: {e}")
        
        return "Hi, I am Power OFF"
    
if __name__ == "__main__":
    # Initialize the DBus main loop first
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    # Start the service
    TestService()

    # Start the GLib main loop
    GLib.MainLoop().run()
